* Instantiate wwDotNetBridge.

set procedure to wwDotNetBridge
loBridge = createobject('wwDotNetBridge', 'V4')

* Load the PowerShell assembly. Get the location using
* [psobject].assembly.location in PowerShell.

llReturn = loBridge.LoadAssembly('C:\Windows\Microsoft.Net\assembly\' + ;
	'GAC_MSIL\System.Management.Automation\v4.0_3.0.0.0__31bf3856ad364e35' + ;
	'\System.Management.Automation.dll')
if not llReturn
	messagebox(loBridge.cErrorMsg)
	return
endif not llReturn

* Create a PowerShell object.

loPS = loBridge.InvokeStaticMethod('System.Management.Automation.PowerShell', ;
	'Create')

* Add a command to execute. You can also call AddScript to add a script.

loBridge.InvokeMethod(loPS, 'AddCommand', 'Get-Process')

* We want to call the Invoke method of the PowerShell object but the following
* statement gives an "ambigous match found" error because there are two
* overloads of Invoke that accept no parameters but with different return
* values and wwDotNetBridge can't tell which to call.
*
* loResult = oBridge.InvokeMethod(loPS, 'Invoke')
*
* Instead, we'll go through the methods of the PowerShell object, look for the
* first Invoke method, and call it. It returns Collection<PSObject>. Note that
* we need an empty array for the second parameter of Invoke.

loType    = loBridge.InvokeMethod(loPS, 'GetType')
loMethods = loBridge.InvokeMethod(loType, 'GetMethods')
loArray   = loBridge.CreateArray()
for lnI = 0 to loMethods.Count - 1
	loMethod = loMethods.Item(lnI)
	if loMethod.Name = 'Invoke'
		loResult = loBridge.InvokeMethod(loMethod, 'Invoke', loPS, loArray)
		exit
	endif loMethod.Name = 'Invoke'
next lnI

* Create a cursor to hold the process information.

create cursor Processes (Name C(50), ID I)

* Copy the result collection to an array and process each entry.

loArray.FromEnumerable(loResult)
for lnI = 0 to loArray.Count - 1
	loItem = loArray.Item(lnI)

* We want to get the ProcessName member of the object but this doesn't work:
* Members is a PSMemberInfoCollection<PSMemberInfo> but accessing it like this
* gives the collection, not the PSPropertyInfo object.
*
* loName    = loBridge.GetPropertyEx(loProcess, 'Members["ProcessName"]')
*
* Instead, get the Members collection and go through it, looking for the
* members we're interested in.

	loMemb    = loBridge.GetPropertyEx(loItem, 'Members')
	loMembers = loBridge.CreateArray()
	loMembers .FromEnumerable(loMemb)
	append blank
	for lnJ = 0 to loMembers .Count
		loMember = loMembers.Item(lnJ)
		try
			lcName = loBridge.GetPropertyEx(loMember, 'Name')
			do case
				case lcName = 'ProcessName'
					replace Name with loBridge.GetPropertyEx(loMember, 'Value')
				case lcName = 'Id'
					replace ID with loBridge.GetPropertyEx(loMember, 'Value')
			endcase
		catch
		endtry
	next lnJ
next lnI

* Display the processes.

go top
browse
